#import "CCWebpageLink.h"

enum WebpageLinkState {
	WLS_REGULAR = 1,
	WLS_HOVER,
	WLS_DOWN
};

@implementation CCWebpageLink
- (void) setUrl:(NSURL*)aUrl
{
	[self setUrl:aUrl withLinkName:[self stringValue]];
}

- (void) setUrl:(NSURL*)aUrl withLinkName:(NSString*)aLinkName
{
	if(isUsernameLink == NO) {
		underlineStyle = NSUnderlineStyleSingle;
		color = [[NSColor blueColor] retain];
		
		underlineHoverStyle = NSUnderlineStyleSingle;
		hoverColor = [[NSColor blueColor] retain];
		
		underlineDownStyle = NSUnderlineStyleSingle;
		downColor = [[NSColor blueColor] retain];
	}
	else {
		underlineStyle = NSUnderlineStyleNone;
		color = [[NSColor blackColor] retain];
		
		underlineHoverStyle = NSUnderlineStyleSingle;
		hoverColor = [[NSColor blackColor] retain];
		
		underlineDownStyle = NSUnderlineStyleSingle;
		downColor = [[NSColor redColor] retain];		
	}

	
	[self setLinkName:aLinkName];
	
	[url release];
	url = [aUrl retain];
}

- (void) setLinkName:(id)aLinkName
{
	[linkName release];
	linkName = [aLinkName retain];

	[self updateStyle:WLS_REGULAR];
}

- (void) updateRects
{
	// create tracking rects
	NSRect rc = [self frame];	
	NSSize stringSize = [[self attributedStringValue] size];
	int xOffset = 0; // NSLeftTextAlignment by default
	
	NSTextAlignment currAlignment = [self getCurrentTextAlignment];
	if(currAlignment == NSRightTextAlignment) {
		if(isUsernameLink) {
			xOffset = ((rc.size.width - 13) - stringSize.width);
			stringSize.width += 10;
		}
		else {
			xOffset = rc.size.width - stringSize.width;
		}
	}
	else if(currAlignment == NSCenterTextAlignment){
		xOffset = (rc.size.width - stringSize.width) / 2;
	}
	
	textRect = NSMakeRect(xOffset, 0, stringSize.width, stringSize.height);
	
	[self removeTrackingRect:rectTag];
	rectTag = [self addTrackingRect:textRect owner:self userData:nil assumeInside:NO];
	
	[[self window] invalidateCursorRectsForView:self];
	[self setPostsBoundsChangedNotifications:YES];
}

- (void) setIsUsernameLink:(BOOL)value
{
	isUsernameLink = value;
}

- (void) awakeFromNib
{
	[self setPostsBoundsChangedNotifications:YES];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(boundsChanged:) name:NSViewBoundsDidChangeNotification object:self];	
}

- (void) boundsChanged:(NSNotification*)aNotification
{
	[self updateRects];
}

- (void) mouseEntered:(NSEvent*)event
{
	isCursorHoveringOverLink = YES;
	[self updateStyle:WLS_HOVER];
}

- (void) mouseExited:(NSEvent*)event
{
	isCursorHoveringOverLink = NO;
	[self updateStyle:WLS_REGULAR];
}

- (void) resetCursorRects
{
    [self addCursorRect:textRect cursor:[NSCursor pointingHandCursor]];
}

- (void) mouseUp:(NSEvent*)event
{
    if(isCursorHoveringOverLink) {
        [[NSWorkspace sharedWorkspace] openURL:url];
		[self updateStyle:WLS_HOVER];
    }
	else {
		[self updateStyle:WLS_REGULAR];
	}
}

- (void) mouseDown:(NSEvent*)event
{
    if(isCursorHoveringOverLink) {
		[self updateStyle:WLS_DOWN];
    }
}

- (void) setStyleWithUnderline:(int)anUnderlineStyle color:(NSColor*)aColor
{
	underlineStyle = anUnderlineStyle;
	[color release];
	color = [aColor retain];
	
	underlineHoverStyle = anUnderlineStyle;	
	[hoverColor release];
	hoverColor = [aColor retain];
	
	underlineDownStyle = anUnderlineStyle;
	[downColor release];
	downColor = [aColor retain];
	
	[self updateStyle:WLS_REGULAR];
}

- (void) setHoverStyleWithUnderline:(int)anUnderlineStyle color:(NSColor*)aColor
{
	underlineHoverStyle = anUnderlineStyle;	
	[hoverColor release];
	hoverColor = [aColor retain];	
}

- (void) setDownStyleWithUnderline:(int)anUnderlineStyle color:(NSColor*)aColor
{
	underlineDownStyle = anUnderlineStyle;
	[downColor release];
	downColor = [aColor retain];
}

- (void) updateStyle:(enum WebpageLinkState)state
{
	int uStyle;
	
	switch(state) {
		case WLS_REGULAR:
			[self setTextColor:color];
			uStyle = underlineStyle;
			break;
			
		case WLS_HOVER:
			[self setTextColor:hoverColor];
			uStyle = underlineHoverStyle;
			break;
			
		case WLS_DOWN:
			[self setTextColor:downColor];
			uStyle = underlineDownStyle;
			break;
			
		default:
			return;
	}

	NSTextAlignment currAlignment = [self getCurrentTextAlignment];
	NSMutableParagraphStyle* paragraphStyle = [[NSMutableParagraphStyle alloc] init];
	[paragraphStyle autorelease];
	[paragraphStyle setLineBreakMode:NSLineBreakByTruncatingTail];
	[paragraphStyle setAlignment:currAlignment];
    
    NSNumber* style = [NSNumber numberWithInt:uStyle];
	NSMutableDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:style, NSUnderlineStyleAttributeName, paragraphStyle, NSParagraphStyleAttributeName, nil];    
	NSMutableAttributedString* str = [[NSMutableAttributedString alloc] initWithString:linkName attributes:attrs];

	if(isUsernameLink == YES) {
		NSRange rng = NSMakeRange(0, 1);
		attrs = [NSDictionary dictionaryWithObjectsAndKeys:style, NSUnderlineStyleAttributeName, paragraphStyle, NSParagraphStyleAttributeName, [NSColor redColor], NSForegroundColorAttributeName, nil];
		[str setAttributes:attrs range:rng];
	}
	
	[self setAttributedStringValue:[str autorelease]];
	
	[self updateRects];
}

- (NSTextAlignment) getCurrentTextAlignment
{
	// get current alingment
	NSAttributedString* currStr = [self attributedStringValue];
	if(currStr) {
		NSDictionary* currAttrs = [currStr attributesAtIndex:0 effectiveRange:nil];
		if(currAttrs) {
			NSMutableParagraphStyle* currParagraphStyle = [currAttrs objectForKey:NSParagraphStyleAttributeName];
			if(currParagraphStyle) {
				return [currParagraphStyle alignment];
			}
		}
	}
	
	return NSLeftTextAlignment;
}

/*
- (void)setTagret:(id)aTarget action:(SEL)anAction
{
    actionObject = object;
    action = NSSelectorFromString(sel);
    [self setColorWithIntR:130 green:130 blue:130];
}

- (void)mouseDown:(NSEvent*)event
{
	if(NSPointInRect([self convertPoint:[event locationInWindow] fromView:nil], [self textRect])) {
		mouseIsDown = YES;
		[self updateStyle];
	}else
		[[self superview] mouseDown:event];
}
*/

/*
- (NSRect)textRect
{
	NSRect textRect = {NSZeroPoint,[[self attributedStringValue] size]};
    
    if(isScreenNameHackEnabled)
        textRect.origin.x += 25;
    
    // arbitrary number to fix odd behavior
    //textRect.size.width += 10;
	return NSIntersectionRect([self bounds], textRect);
}

- (void) awakeFromNib
{
	[self setPostsBoundsChangedNotifications:YES];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(boundsChanged:) name:NSViewBoundsDidChangeNotification object:self];
	rectTag = [self addTrackingRect:[self textRect] owner:self userData:nil assumeInside:NO];
	
    color = [[NSColor blueColor] retain];
    [self updateStyle];
    
    isScreenNameHackEnabled = NO;
}

- (void)updateRects
{
	[self removeTrackingRect:rectTag];
	rectTag = [self addTrackingRect:[self textRect] owner:self userData:nil assumeInside:NO];
	[[self window] invalidateCursorRectsForView:self];
}

- (void)boundsChanged:(NSNotification*)not
{
	[self updateRects];
}

- (void)setObjectValue:(id)string callbackObject:(id)object selector:(NSString*)sel
{    
    actionObject = object;
    action = NSSelectorFromString(sel);
    [self setObjectValue:string];
}

- (void)setObjectValue:(id)string
{
    if(isScreenNameHackEnabled)
        [super setObjectValue:[NSString stringWithFormat:@"Вы: %@", string]];
    else
        [super setObjectValue:string];
    
	NSMutableParagraphStyle* style = [[NSMutableParagraphStyle alloc] init];
	[style autorelease];
	[style setLineBreakMode:NSLineBreakByTruncatingTail];
    
    NSNumber* underlineStyle = [NSNumber numberWithInt:NSUnderlineStyleSingle];
    
    //if link is not weblink, but performs some action, make dotted underline
    if(action)
        underlineStyle = [NSNumber numberWithInt:NSUnderlineStyleSingle | NSUnderlinePatternDot];
    
	NSMutableDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:underlineStyle, NSUnderlineStyleAttributeName, style, NSParagraphStyleAttributeName, nil];
    
	NSMutableAttributedString* str = [[NSMutableAttributedString alloc] initWithString:[self stringValue] attributes:attrs];
	[str autorelease];
	[self setAttributedStringValue:str];
	[self updateRects];
    
    if(isScreenNameHackEnabled)
    {
        // remove underlining on the whole string
        NSRange rng = NSMakeRange(0, [str length]);
        attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleNone], NSUnderlineStyleAttributeName, nil];
        [str setAttributes:attrs range:rng];
        
        //set underlining skipping "Вы: "
        rng = NSMakeRange(4, [str length] - 4);
        attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleSingle], NSUnderlineStyleAttributeName, nil];
        [str setAttributes:attrs range:rng];
        
        rng = NSMakeRange(4, 1);
        attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleSingle], NSUnderlineStyleAttributeName, [NSColor redColor], NSForegroundColorAttributeName, nil];
        [str setAttributes:attrs range:rng];
        
        //size to fit and align control to the right
        NSRect fr = [self frame];
        int prevWidth = fr.size.width;
        
        [self sizeToFit];
        
        fr = [self frame];
        int delta = prevWidth - fr.size.width;
        fr.origin.x += delta;
        [self setFrame:fr];
        
        [self setAttributedStringValue:str];
        [self updateRects];
    }
}

- (void)setIsScreenNameHackEnabled:(BOOL)hackOk
{
    isScreenNameHackEnabled = hackOk;
}

- (void)resetCursorRects
{
	[self addCursorRect:[self textRect] cursor:[NSCursor pointingHandCursor]];
}

- (void)updateStyle
{
    if(isScreenNameHackEnabled)
    {
        NSMutableAttributedString* str = [[NSMutableAttributedString alloc] init];
        [str setAttributedString:[self attributedStringValue]];
        
        if(mouseIsDown)
        {
            if(mouseIsWithin)
            {
                NSRange rng = NSMakeRange(4, [str length] - 4);
                NSDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleSingle], NSUnderlineStyleAttributeName, [NSColor redColor], NSForegroundColorAttributeName, nil];
                [str setAttributes:attrs range:rng];
            }
        }
        else
        {
            NSRange rng = NSMakeRange(0, [str length]);
            NSDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleSingle], NSUnderlineStyleAttributeName, [NSColor blackColor], NSForegroundColorAttributeName, nil];
            [str setAttributes:attrs range:rng];
            
            attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleSingle], NSUnderlineStyleAttributeName, [NSColor redColor], NSForegroundColorAttributeName, nil];
            rng = NSMakeRange(4, 1);
            [str setAttributes:attrs range:rng];
            
            attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:NSUnderlineStyleNone], NSUnderlineStyleAttributeName, [NSColor blackColor], NSForegroundColorAttributeName, nil];
            rng = NSMakeRange(0, 4);
            [str setAttributes:attrs range:rng];
        }
        
        [self setAttributedStringValue:str];
    }
    else
    {
        if(action)
        {       
            [color release];
            color = [[NSColor colorWithDeviceRed:(90.0f / 255.0f) green:(90.0f / 255.0f) blue:(90.0f / 255.0f) alpha:1] retain];
        }
        
        //NSColor* textColor = mouseIsDown && mouseIsWithin ? [NSColor redColor] : color;
        NSColor* textColor = mouseIsDown ? [NSColor redColor] : color;
        [self setTextColor:textColor];
    }
}

- (void)mouseUp:(NSEvent*)event
{
	mouseIsDown = NO;
    
	//if(mouseIsWithin)
    //{
    if(actionObject && action)
        [actionObject performSelector:action];
    else
        [[NSWorkspace sharedWorkspace] openURL:url];
    //}
    
	[self updateStyle];
	[[self window] invalidateCursorRectsForView:self];
}

- (void)setCallbackObject:(id)object selector:(NSString*)sel
{
    actionObject = object;
    action = NSSelectorFromString(sel);
    [self setColorWithIntR:130 green:130 blue:130];
}

- (void)mouseDown:(NSEvent*)event
{
	if(NSPointInRect([self convertPoint:[event locationInWindow] fromView:nil], [self textRect])) {
		mouseIsDown = YES;
		[self updateStyle];
	}else
		[[self superview] mouseDown:event];
}

- (void)setColorWithFloatR:(float)r green:(float)g blue:(float)b
{
    [color autorelease];
    color = [[NSColor colorWithDeviceRed:r green:g blue:b alpha:1] retain];
    
    [self updateStyle];
}

- (void)setColorWithIntR:(int)r green:(int)g blue:(int)b
{
    [color release];
    color = [[NSColor colorWithDeviceRed:(r / 255.0f) green:(g / 255.0f) blue:(b / 255.0f) alpha:1] retain];
    
    [self updateStyle];
}

- (void)mouseEntered:(NSEvent*)event
{
	//mouseIsWithin = YES;
	[self updateStyle];
}

- (void)mouseExited:(NSEvent*)event
{
	//mouseIsWithin = NO;
	[self updateStyle];
}

- (NSURL*) url
{
	return url;
}

- (void) setUrl:(NSURL*)u
{
	if(url)
		[url release];
    
	url = [[NSURL URLWithString:[u absoluteString]] retain];
}
*/
- (void) dealloc
{
    [url release];
	[color release];
	[hoverColor release];
	[downColor release];
	[linkName release];
    
	[super dealloc];
}

@end