//
//  AlbumsOutlineViewController.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 10.01.10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "AlbumsOutlineViewController.h"
#import "AlbumsOutlineViewItem.h"
#import "ImageAndTextCell.h"

NSString* const YafAlbumsViewSelectionChangedNotification = @"YafAlbumsViewSelectionChangedNotification";

@implementation AlbumsOutlineViewController
- (void) awakeFromNib
{
	NSString* iconPath = [[NSBundle mainBundle] pathForResource:@"smooth_folder_blue" ofType:@"icns"];
	albumIcon = [[NSImage alloc] initWithContentsOfFile:iconPath];
	
	iconPath = [[NSBundle mainBundle] pathForResource:@"locked_album" ofType:@"icns"];
	albumIconLocked = [[NSImage alloc] initWithContentsOfFile:iconPath];
}

// Data Source methods

- (NSInteger) outlineView:(NSOutlineView*)outlineView numberOfChildrenOfItem:(id)item {
    return (item == nil) ? 1 : [item numberOfChildren];
}

- (BOOL) outlineView:(NSOutlineView*)outlineView isItemExpandable:(id)item {
    return (item == nil) ? YES : ([item numberOfChildren] != 0);
}

- (id) outlineView:(NSOutlineView*)outlineView child:(NSInteger)index ofItem:(id)item {
	if(item == nil) {
		return [AlbumsOutlineViewItem rootItem];
	}

    return [(AlbumsOutlineViewItem*)item childAtIndex:index];
}

- (id) outlineView:(NSOutlineView*)outlineView objectValueForTableColumn:(NSTableColumn*)tableColumn byItem:(id)item {
    return [(AlbumsOutlineViewItem*)item description];
}

// Delegate methods

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn *)tableColumn item:(id)item {
    return NO;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldCollapseItem:(id)item
{
	//if(![item parent]) {
	//	return NO;
	//}
	
	return YES;
}

- (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	// if is album item
	if([item isKindOfClass:[AlbumsOutlineViewItem class]] && [(AlbumsOutlineViewItem*)item parent]) {
		id itemObject = [(AlbumsOutlineViewItem*)item object];
						 
		if([itemObject isKindOfClass:[YafAlbum class]]) {
			NSImage* image = albumIcon;
			
			if([(YafAlbum*)itemObject isProtected] == YES) {
				image = albumIconLocked;
			}
			
			NSSize sz = {16, 16};
			[image setSize:sz];
			[cell setImage:image];
		}
	}
	else {
		[cell setImage:nil];
	}
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	[self updateAlbumsSelectionInfo];
}

- (void) updateAlbumsSelectionInfo
{
	if(!yafUser) {
		return;
	}
	
	NSArray* albums = [yafUser albums];
	int selectedNodesCount = [[albumsView selectedRowIndexes] count];
	
	selectedAlbumsCount = -1;
	selectedImagesCount = 0;
	
	// if nothing selected -- get totals
	if(selectedNodesCount == 0) {
		selectedAlbumsCount = [albums count];
		
		for(int i = 0; i < [albums count]; i++) {
			selectedImagesCount += [(YafAlbum*)[albums objectAtIndex:i] imageCount];
		}
	}
	else if(selectedNodesCount == 1) {
		NSInteger row = [albumsView selectedRow];
		AlbumsOutlineViewItem* item = [albumsView itemAtRow:row];
		
		if([item parent] == nil) { // if root
			selectedAlbumsCount = [albums count];
			
			for(int i = 0; i < [albums count]; i++) {
				selectedImagesCount += [(YafAlbum*)[albums objectAtIndex:i] imageCount];
			}
		}
		else { // if album
			selectedImagesCount = [(YafAlbum*)[item object] imageCount];
		}
	}
	else {
		// iterate through selected nodes
		NSIndexSet* changedSel = [albumsView selectedRowIndexes];
		NSUInteger currentIndex = [changedSel firstIndex];
		while(currentIndex != NSNotFound)
		{
			AlbumsOutlineViewItem* item = [albumsView itemAtRow:currentIndex];
			if([item parent] != 0) {
				if(selectedAlbumsCount == -1) {
					selectedAlbumsCount = 0;
				}
				
				// count selected albums
				selectedAlbumsCount++;
				
				// count images in selected albums
				selectedImagesCount += [(YafAlbum*)[item object] imageCount];
			}
			
			currentIndex = [changedSel indexGreaterThanIndex:currentIndex];
		}
	}
	
	NSNotification* myNotification = [NSNotification notificationWithName:YafAlbumsViewSelectionChangedNotification object:nil];
	[[NSNotificationQueue defaultQueue] enqueueNotification:myNotification postingStyle:NSPostWhenIdle coalesceMask:NSNotificationCoalescingOnName forModes:nil];
}

- (void) setYafUser:(YafUser*)user
{
	[yafUser release];
	yafUser = [user retain];
	[yafUser removeObserver:self];
	[yafUser addObserver:self];
	
	AlbumsOutlineViewItem* root = [AlbumsOutlineViewItem rootItem];
	[root setYafUser:yafUser];
}

- (int) selectedAlbumsCount
{
	return selectedAlbumsCount;
}

- (int) selectedImagesCount
{
	return selectedImagesCount;
}

- (void) yandexFotkiObjectSyncronized:(YaFotkiOperationType)operationType operationData:(NSDictionary*)operationData error:(NSError*)error
{
	if(operationType == YFO_GET_ALBUMS_LIST) {
		if(!error) {
			[[AlbumsOutlineViewItem rootItem] attachAlbums];
			[albumsView reloadData];
			[albumsView expandItem:nil expandChildren:YES];
			
			ImageAndTextCell *browserCell = [[[ImageAndTextCell alloc] init] autorelease];
			//[browserCell setLeaf:YES];
			[[[albumsView tableColumns] objectAtIndex:0] setDataCell:browserCell];
			[self updateAlbumsSelectionInfo];
		}
	}
}
@end
