#import "SettingsManager.h"

static SettingsManager* instance = nil;

@implementation SettingsManager
- (id) init
{
	if(self = [SettingsManager alloc]) {
		username = @"";
		password = @"";
		shouldSignInOnStartup = NO;
		
		// current preferences version
		configVersion = CURRENT_PREFERENCES_FILE_VERSION;
	}
	
	return self;
}

+ (id) alloc
{
	if(instance == nil)
		instance = [super alloc];
	
	return instance;
}

+ (SettingsManager*) instance
{
	if(!instance)
		instance = [[SettingsManager alloc] init];
	
	return instance;
}

- (int) configVersion
{
	return configVersion;
}

- (void) setConfigVersion:(int)value
{
	configVersion = value;
	[self saveToFile];
}

- (void) loadFromFile
{
    NSString* path = PREFERENCES_PATH;
	path = [path stringByExpandingTildeInPath];
    NSFileHandle* preferencesFile = [NSFileHandle fileHandleForWritingAtPath:path];
    
    // if no preferences file found save defaults
    if(!preferencesFile) {
        [self saveToFile];
        return;
    }
		
    NSMutableDictionary* prefs = [[NSMutableDictionary alloc] initWithContentsOfFile:path];
    [prefs autorelease];
	
	BOOL isParamMissing = NO;
	
    // load username
    if([prefs objectForKey:@"username"]) {
        [username release];
		username = [[prefs objectForKey:@"username"] retain];
    }
    else {
		isParamMissing = YES;
	}
	
    // load shouldSignInOnStartup
    if([prefs objectForKey:@"shouldSignInOnStartup"]) {
        shouldSignInOnStartup = [[prefs objectForKey:@"shouldSignInOnStartup"] boolValue];
    }
    else {
		isParamMissing = YES;
	}
	
	NSLog(@"%@", username);
	NSLog(@"%@", password);
	
 	if(isParamMissing) {
		[self saveToFile];
	}
}

- (void) saveToFile
{
    NSString* path = PREFERENCES_PATH;
	path = [path stringByExpandingTildeInPath];
    NSMutableDictionary* prefs = [[NSMutableDictionary alloc] initWithContentsOfFile:path];
	
	// if no existing data —- create new
	if(!prefs)
		prefs = [[NSMutableDictionary alloc] init];

	[prefs setObject:username forKey:@"username"];
	[prefs setObject:[NSNumber numberWithBool:shouldSignInOnStartup] forKey:@"shouldSignInOnStartup"];
	
	// save file's format version
	[prefs setObject:[NSNumber numberWithInt:configVersion] forKey:@"configVersion"];
	
    // save prefs data to the user's home directory/Library/Preferences.
    [prefs writeToFile:path atomically:TRUE];
	[prefs release];
}

- (NSString*) username
{
	return username;
}

- (void) setUsername:(NSString*)value
{
	[username release];
	username = [value copy];
	[self saveToFile];
}

- (NSString*) password
{
	return [password stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
}

- (void) setPassword:(NSString*)value
{
	[password release];
	password = [[value stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] copy];
}

- (BOOL) shouldSignInOnStartup
{
	return shouldSignInOnStartup;
}

- (void) setShouldSignInOnStartup:(BOOL)value
{
	shouldSignInOnStartup = value;
	[self saveToFile];
}

- (void) savePasswordInKeychain
{
	if(!username || [username length] == 0) {
		return;
	}
	
	[self removePasswordFromKeychain];
	
	NSString* appName = APP_NAME;
	
	SecKeychainAddGenericPassword(NULL, 
								  [appName length],
								  [appName UTF8String],
								  [username length],
								  [username UTF8String],
								  [password length],
								  [password UTF8String],
								  NULL);
}

- (void) loadPasswordFromKeychain
{
	if(!username || [username length] == 0) {
		return;
	}
	
	NSString* appName = APP_NAME;
	
	// load password from keychain for current username
	void* passwordData;
	UInt32 passwordLength;
	OSStatus status = SecKeychainFindGenericPassword(
													 NULL,                  // default keychain
													 [appName length],       // length of service name
													 [appName UTF8String],   // service name
													 [username length],
													 [username UTF8String],
													 &passwordLength,       // length of password
													 &passwordData,         // pointer to password data
													 NULL);
	
	// pass found
	if(status == noErr)
	{
		// try to login with given credentials
		[password release];
		password = [NSString stringWithCString:(const char*)passwordData encoding:NSUTF8StringEncoding];
		NSLog(@"psss 0: %@", password);
		password = [password substringToIndex:passwordLength];
		[password retain];
		SecKeychainItemFreeContent(NULL, passwordData); // release password data buffer
	}
	
	NSLog(@"psss: %@", password);
}

- (void) removePasswordFromKeychain
{
	if(!username || [username length] == 0) {
		return;
	}
	
	NSString* appName = APP_NAME;
	SecKeychainItemRef itemRef;
	OSStatus status = SecKeychainFindGenericPassword(
													 NULL,					// default keychain
													 [appName length],       // length of service name
													 [appName UTF8String],   // service name
													 [username length],
													 [username UTF8String],
													 NULL,					// length of password
													 NULL,					// pointer to password data
													 &itemRef);
	
	// pass found
	if(status == noErr)
	{
		SecKeychainItemDelete(itemRef);
		CFRelease(itemRef);
	}
}
@end