//
//  SignInWindowController.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 09.01.10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "SignInWindowController.h"
#import "SettingsManager.h"
#import "InputFormatter.h"

NSString* const YafSignInSuccessfulNotification = @"YafSignInSuccessfulNotification";

@implementation SignInWindowController
- (void) awakeFromNib
{
	[forgotPassword setUrl:[NSURL URLWithString:@"http://passport.yandex.ru/passport?mode=remember"]];
	[forgotPassword setStyleWithUnderline:NSUnderlineStyleNone color:[NSColor whiteColor]];
	[forgotPassword setHoverStyleWithUnderline:NSUnderlineStyleSingle color:[NSColor whiteColor]];
	[forgotPassword setDownStyleWithUnderline:NSUnderlineStyleSingle color:[NSColor redColor]];

	[window setOpaque:NO];
	//NSColor *semiTransparentBlue = [NSColor colorWithDeviceRed:0.495 green:0.666 blue:0.0 alpha:1.0];
	NSColor *semiTransparentBlue = [NSColor colorWithDeviceRed:56/255.0f green:127/255.0f blue:181/255.0f alpha:1.0];
	[window setBackgroundColor:semiTransparentBlue];
	
	NSButton* minButton = [window standardWindowButton:NSWindowMiniaturizeButton];
	[minButton setHidden:YES];
	
	NSButton* zoomButton = [window standardWindowButton:NSWindowZoomButton];
	[zoomButton setHidden:YES];

	InputFormatter* formatter = [[InputFormatter alloc] initWithMaxLength:30];
	[formatter autorelease];
	
	[login setFormatter:formatter];
	[password setFormatter:formatter];
	
	if([SettingsMgr username] && [[SettingsMgr username] length] != 0 && [[SettingsMgr username] length] < 30) {
		[login setStringValue:[SettingsMgr username]];
		[window setInitialFirstResponder:password];
	}
	
	if([SettingsMgr shouldSignInOnStartup]) {
		[shouldRememberUser setState:NSOnState];
	}
	else {
		[shouldRememberUser setState:NSOffState];
	}
	
	[self restoreDefaultWindowSize:NO];
}

- (void)windowWillClose:(NSNotification *)notification
{
	[yafUser removeObserver:self];
	[NSApp terminate:self];
}

- (void) showWindow:(BOOL)doShow
{
	[window setIsVisible:doShow];
}

- (void) signInWithUsername:(NSString*)aUsername password:(NSString*)aPassword
{
	NSString* loginStr = [aUsername stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	NSString* passwordStr = [aPassword stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	
	if(!loginStr || [loginStr length] == 0 || [loginStr length] > 30 ||
	   !passwordStr || [passwordStr length] == 0 || [passwordStr length] > 30) {
		[tabView selectTabViewItemAtIndex:0];
		[window setIsVisible:YES];
		//[self showSignInFailureTooltip];
		return;
	}

	[tabView selectTabViewItemAtIndex:1];
	[progress startAnimation:self];
	[window setIsVisible:YES];
	
	[yafUser signInWithUsername:aUsername password:aPassword];
}

- (void) setYafUser:(YafUser*)user
{
	[yafUser release];
	yafUser = [user retain];
	[yafUser removeObserver:self];
	[yafUser addObserver:self];
}

- (IBAction) signIn:(id)sender
{
	NSString* loginStr = [login stringValue];
	NSString* passwordStr = [password stringValue];
	
	loginStr = [loginStr stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	passwordStr = [passwordStr stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];

	if(!loginStr || [loginStr length] == 0 || [loginStr length] > 30 ||
	   !passwordStr || [passwordStr length] == 0 || [passwordStr length] > 30) {
		[self showSignInFailureTooltip];
		return;
	}
	
	[SettingsMgr setUsername:loginStr];
	
	[self restoreDefaultWindowSize:YES];
	[tabView selectTabViewItemAtIndex:1];
	[progress startAnimation:self];
	[yafUser signInWithUsername:loginStr password:passwordStr];
}

- (void) restoreDefaultWindowSize:(BOOL)doAnimate
{
	[tooltip setHidden:YES];
	
	int windowHeight = 300;
	NSRect frame = [window frame];
    int oldHeight = frame.size.height;
    frame.size.height = windowHeight + 25;
    frame.origin.y -= frame.size.height - oldHeight;
    
    [window setFrame:frame display:YES animate:doAnimate];
}

- (void) showSignInFailureTooltip
{
	[self restoreDefaultWindowSize:YES];
	
	[tooltip setHidden:NO];
	
	NSString* errorTitle = @"Неправильная пара логин-пароль!\nАвторизоваться не удалось.";
	NSString* errorDescription = @"Проверьте раскладку клавиатуры, не нажата ли клавиша «Caps Lock» и попробуйте ввести логин и пароль еще раз.";
	
	NSRect rcTooltip = [tooltip frame];
	rcTooltip.origin.y = 90;
	rcTooltip.size.height = 131;
	[tooltip setFrame:rcTooltip];

	NSRect rcTitle = [errorHeaderText frame];
	rcTitle.size.height = 28;
	rcTitle.origin.y = 80;
	[errorHeaderText setFrame:rcTitle];

	NSRect rcDescr = [errorMessageText frame];
	rcDescr.size.height = 60;
	rcDescr.origin.y = 10;
	[errorMessageText setFrame:rcDescr];
	
	[errorHeaderText setStringValue:errorTitle];
	[errorMessageText setStringValue:errorDescription];
	
	int windowHeight = 430;
	NSRect frame = [window frame];
    int oldHeight = frame.size.height;
    frame.size.height = windowHeight + 25;
    frame.origin.y -= frame.size.height - oldHeight;
    
    [window setFrame:frame display:YES animate:YES];
}

- (void) showServerUnavailableTooltip
{
	[self restoreDefaultWindowSize:YES];
	
	[tooltip setHidden:NO];
	
	NSString* errorTitle = @"Ошибка подключения.\nСервер недоступен.";
	NSString* errorDescription = @"Проверьте подключение к интернету, или повторите попытку позже.";
	
	NSRect rcTooltip = [tooltip frame];
	rcTooltip.origin.y = 120;
	rcTooltip.size.height = 101;
	[tooltip setFrame:rcTooltip];
	
	NSRect rcTitle = [errorHeaderText frame];
	rcTitle.size.height = 28;
	rcTitle.origin.y = 50;
	[errorHeaderText setFrame:rcTitle];
	
	NSRect rcDescr = [errorMessageText frame];
	rcDescr.size.height = 60;
	rcDescr.origin.y = -20;
	[errorMessageText setFrame:rcDescr];
	
	[errorHeaderText setStringValue:errorTitle];
	[errorMessageText setStringValue:errorDescription];
	
	int windowHeight = 400;
	NSRect frame = [window frame];
    int oldHeight = frame.size.height;
    frame.size.height = windowHeight + 25;
    frame.origin.y -= frame.size.height - oldHeight;
    
    [window setFrame:frame display:YES animate:YES];
}

- (IBAction) rememberUserSwitch:(id)sender
{
	if([shouldRememberUser state] == NSOffState) {
		[SettingsMgr setShouldSignInOnStartup:NO];
	}
	else {
		[SettingsMgr setShouldSignInOnStartup:YES];
	}
}

- (void) yandexFotkiObjectSyncronized:(YaFotkiOperationType)operationType operationData:(NSDictionary*)operationData error:(NSError*)error
{
	if(!error) {
		if(operationType == YFO_SIGN_IN) {
			if([SettingsMgr shouldSignInOnStartup]) {
				[SettingsMgr setPassword:[yafUser password]];
				[SettingsMgr savePasswordInKeychain];
			}
			
			// get user albums and all other data
			[yafUser populateAlbumsList];
		}
		else if(operationType == YFO_GET_ALBUMS_LIST) {
			[self showWindow:NO];
			NSNotification* myNotification = [NSNotification notificationWithName:YafSignInSuccessfulNotification object:nil];
			[[NSNotificationQueue defaultQueue] enqueueNotification:myNotification postingStyle:NSPostWhenIdle coalesceMask:NSNotificationCoalescingOnName forModes:nil];			
		}
	}
	else { // something went wrong
		if([error code] == YafSignInfailureErrorType) {
			[NSTimer scheduledTimerWithTimeInterval:2.0 target:self selector:@selector(onTimerSignInFailure:) userInfo:nil repeats:NO];
		}
		else if([error code] == YafServerUnreachableErrorType) {
			[NSTimer scheduledTimerWithTimeInterval:2.0 target:self selector:@selector(onTimerServerFailure:) userInfo:nil repeats:NO];
		}
	}
}

- (void) showLoginPage
{
	[self restoreDefaultWindowSize:NO];
	[tabView selectTabViewItemAtIndex:0];
	[password setStringValue:@""];
	[shouldRememberUser setState:NSOffState];
	[SettingsMgr setShouldSignInOnStartup:NO];
	[SettingsMgr removePasswordFromKeychain];
}

- (void) onTimerSignInFailure:(NSTimer*)timer
{
	[tabView selectTabViewItemAtIndex:0];
	[progress stopAnimation:self];
	[self showSignInFailureTooltip];
}

- (void) onTimerServerFailure:(NSTimer*)timer
{
	[tabView selectTabViewItemAtIndex:0];
	[progress stopAnimation:self];
	[self showServerUnavailableTooltip];	
}
@end