//
//  YaFotkiOperation.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 23.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YaOperation.h"

NSString* const YafErrorDomain = @"YafAPIErrorDomain";

@implementation YaOperation
- (id) init
{
	return [self initWithCatcher:nil];
}

- (id) initWithCatcher:(id<YaOperationCatcher>)aCatcher
{
	if(self = [super init]) {
		[self setCatcher:aCatcher];
		
		// create empty queue for requests
		requestsQueue = [[NSMutableArray alloc] init];
		
		NSDictionary* errDic = nil;
		
		errDic = [NSDictionary dictionaryWithObjectsAndKeys:@"Сервер недоступен, пожалуйста, попробуйте повторить попытку позже.", NSLocalizedDescriptionKey, nil];
		serverUnreacheableError = [NSError errorWithDomain:YafErrorDomain code:YafServerUnreachableErrorType userInfo:errDic];
		[serverUnreacheableError retain];
		
		errDic = [NSDictionary dictionaryWithObjectsAndKeys:@"Неправильная пара логин/пароль.", NSLocalizedDescriptionKey, nil];
		signInError = [NSError errorWithDomain:YafErrorDomain code:YafSignInfailureErrorType userInfo:errDic];
		[signInError retain];

		errDic = [NSDictionary dictionaryWithObjectsAndKeys:@"Внутреняя ошибка, пожалуйста обратитесь в поддержку.", NSLocalizedDescriptionKey, nil];
		yafApiError = [NSError errorWithDomain:YafErrorDomain code:YafSignInfailureErrorType userInfo:errDic];
		[yafApiError retain];
	}
	
	return self;
}

- (void) setCatcher:(id<YaOperationCatcher>)aCatcher
{
	if(aCatcher != nil) {
		[(NSObject*)catcher release];
		catcher = [(NSObject*)aCatcher retain];
	}
	else {
		NSLog(@"%@ has no designated catcher!", [self className]);
	}
}

- (BOOL) isSuccess
{
	return isSuccess;
}

- (void) start
{
	if(!requestsQueue || [requestsQueue count] == 0) {
		return;
	}
	
	if(!catcher) {
		NSLog(@"%@ has no designated catcher!", [self className]);
	}
	
	[self resetOperationData];
	
	// run first request in a queue
	currentRequest = [requestsQueue objectAtIndex:currentRequestIndex];
	[currentRequest start];
}

- (void) resetOperationData
{
	[error release];
	error = nil;
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
}

- (void) requestFailed:(YaRequest *)aRequest
{
	NSString* errorDetails = [NSString stringWithFormat:@"%@ failed. details: %@. error: %@", [aRequest className], aRequest, [aRequest errorMessage]];
	
	isSuccess = NO;
	[self setDetailedErrorMessage:errorDetails];
	[self setError:serverUnreacheableError];
	
	[catcher operationFinished:self];
}

- (NSString*) detailedErrorMessage
{
	return detailedErrorMessage;
}

- (void) setDetailedErrorMessage:(NSString*)anErrorMessage
{
	[detailedErrorMessage release];
	detailedErrorMessage = [anErrorMessage copy];
}

- (NSError*) error
{
	return error;
}

- (void) setError:(NSError*)anError
{
	[error release];
	error = [anError retain];
}

- (void) dealloc
{
	[requestsQueue release];
	[detailedErrorMessage release];

	[serverUnreacheableError release];
	[signInError release];
	[yafApiError release];
	
	[error release];

	[super dealloc];
}
@end
