#import <Cocoa/Cocoa.h>
#import "ASIHTTPRequest.h"
#import "YaRequestParser.h"

enum YaFotkiRequestType {
	YFR_GET,
	YFR_POST,
	YFR_DELETE,
	YFR_PUT,
	YFR_HEAD
};

@class YaRequest;

@protocol YaRequestCatcher
- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary*)parsedData;
- (void) requestFailed:(YaRequest *)aRequest;
@end

@interface YaRequest : NSObject {
	ASIHTTPRequest *request;
	enum YaFotkiRequestType type;

	id<YaRequestCatcher> catcher;
	id<YaRequestParser> parser;
	
	NSString* localErrorMessage;
}

// main utility methods
- (id) initPostRequest;
- (id) initGetRequest;
- (id) initDeleteRequest;
- (id) initPutRequest;
- (id) initHeadRequest;
- (void) start;
- (void) stop;

// getters-setters
- (enum YaFotkiRequestType) type;
- (void) setCatcher:(id<YaRequestCatcher>)aCatcher;
- (void) setParser:(id<YaRequestParser>)aParser;
- (NSString*) description;
- (int) responseCode;
- (NSString*) errorMessage;
- (NSString*) urlString;

// class factories
+ (YaRequest *) createGetRequest;
+ (YaRequest *) createPostRequest;
@end
