#import "YaRequest.h"

@interface YaRequest(Private)
	- (BOOL) isInputDataOk;
@end

@implementation YaRequest
- (id) init
{
	return [self initGetRequest];
}

- (void) setCatcher:(id<YaRequestCatcher>)aCatcher
{
	if(aCatcher != nil) {
		[(NSObject*)catcher release];
		catcher = [(NSObject*)aCatcher retain];
	}
	else {
		NSLog(@"Warning: %@ has no designated catcher.", [self className]);
	}
}

- (void) setParser:(id<YaRequestParser>)aParser
{		
	if(aParser != nil) {
		[(NSObject*)parser release];
		parser = [(NSObject*)aParser retain];
	}
	else {
		NSLog(@"Warning: %@ has no designated parser.", [self className]);
	}	
}

- (id) initPostRequest
{
	if(self = [super init]) {
		request = [[ASIHTTPRequest requestWithURL:nil] retain];
		[request setRequestMethod:@"POST"];
		type = YFR_POST;
	}
	
	return self;
}

- (id) initGetRequest
{
	if(self = [super init]) {
		request = [[ASIHTTPRequest requestWithURL:nil] retain];
		[request setRequestMethod:@"GET"];
		type = YFR_GET;
	}
	
	return self;
}

- (id) initDeleteRequest
{
	if(self = [super init]) {
		request = [[ASIHTTPRequest requestWithURL:nil] retain];
		[request setRequestMethod:@"DELETE"];
		type = YFR_DELETE;
	}
	
	return self;
}

- (id) initPutRequest
{
	if(self = [super init]) {
		request = [[ASIHTTPRequest requestWithURL:nil] retain];
		[request setRequestMethod:@"PUT"];
		type = YFR_PUT;
	}
	
	return self;
}

- (id) initHeadRequest
{
	if(self = [super init]) {
		request = [[ASIHTTPRequest requestWithURL:nil] retain];
		[request setRequestMethod:@"HEAD"];
		type = YFR_HEAD;
	}
	
	return self;
}

+ (YaRequest *) createGetRequest
{
	YaRequest *retVal = [[YaRequest alloc] initGetRequest];
	return [retVal autorelease];
}

+ (YaRequest *) createPostRequest
{
	YaRequest *retVal = [[YaRequest alloc] initPostRequest];
	return [retVal autorelease];
}

- (enum YaFotkiRequestType) type
{
	return type;
}

- (void) start
{
	if(request != nil) {
		[request setDelegate:self];
		[request startAsynchronous];
	}
}

- (void) stop
{
	if([request isFinished] == NO) {
		[request cancel];
	}	
}

// main network events handlers
- (void) requestFinished:(ASIHTTPRequest *)aRequest
{
	NSDictionary* parsedData = nil;
	
	if(parser) {
		parsedData = [parser parseData:[aRequest responseData]];
	}
	
	[catcher requestFinished:self parsedData:parsedData];
}

- (void) requestFailed:(ASIHTTPRequest *)aRequest
{
	[catcher requestFailed:self];
}

- (NSString*) description
{
	NSString* typeAsString = @"GET";
	
	if(type == YFR_POST) {
		typeAsString = @"POST";
	}

	return [NSString stringWithFormat:@"%@, type: %@, %@", [self className], typeAsString, [self urlString]];
}

- (int) responseCode
{
	return [request responseStatusCode];
}

- (NSString*) errorMessage
{
	NSMutableString* localizedErrorDescription = [[[NSMutableString alloc] init] autorelease];
	
	if(localErrorMessage) {
		return localErrorMessage;
	}
	
	if(request) {
		[localizedErrorDescription appendFormat:@"response code: %d, ", [request responseStatusCode]];
		
		if([[[request error] localizedDescription] length] > 0) {
			[localizedErrorDescription appendString:[[request error] localizedDescription]];
		}
																				  
		if([[request error] code] == ASIConnectionFailureErrorType) {
			NSString* errDesc = [[[request error] userInfo] objectForKey:NSUnderlyingErrorKey];
			[localizedErrorDescription appendFormat:@", %@", errDesc];
		}
	}
	
	return localizedErrorDescription;
}

- (NSString*) urlString
{
	if(request) {
		return [[request url] absoluteString];
	}
	
	return nil;
}

// this method should be overridden in derived objects
- (BOOL) isInputDataOk
{
	return YES;
}

- (void) dealloc
{
	[self stop];
	[request release];
	[localErrorMessage release];
	
	[super dealloc];
}
@end
