//
//  YaFotkiUser.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 24.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafUser.h"

NSString* const YafFetchedAlbum = @"YafFetchedAlbum";

@implementation YafUser
- (id) init
{
	if(self = [super init]) {
	}
	
	return self;
}

- (void) addObserver:(id<YafCatcher>)observer
{
	if(!observer) {
		return;
	}
	
	if(!observers) {
		observers = [[NSMutableArray alloc] init];
	}
	
	[observers addObject:observer];
}

- (void) removeObserver:(id<YafCatcher>)observer
{
	if(!observers || [observers count] == 0) {
		return;
	}
	
	int i;
	for(i = 0; i < [observers count]; i++) {
		if([[observers objectAtIndex:i] isEqualTo:observer]) {
			[observers removeObjectAtIndex:i];
			return;
		}
	}
}

- (void) signInWithUsername:(NSString*)aUsername password:(NSString*)aPassword
{
	[signInOperation release];
	signInOperation = [[YafSignInOperation alloc] initWithCatcher:self];
	
	[username release];
	username = [aUsername retain];

	[password release];
	password = [aPassword retain];
	
	[signInOperation signInWithLogin:username password:password];
}

- (void) populateAlbumsList
{
	[getAlbumsListOperation release];
	getAlbumsListOperation = [[YafGetAlbumsListOperation alloc] initWithCatcher:self];
	[getAlbumsListOperation setUsername:username withToken:token];
	[getAlbumsListOperation getAlbumsList];
}

- (void) updateAlbumWithID:(NSString*)albumID title:(NSString*)aTitle summary:(NSString*)aSummary password:(NSString*)aPassword
{
	YafAlbum* album = [[self getAlbumWithID:albumID] copy];
	[album autorelease];
	
	if(!album) {
		return;
	}
	
	aTitle = [aTitle stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	aSummary = [aSummary stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	aPassword = [aPassword stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];

	// title length can not be zero
	if(aTitle) {
		if([aTitle length] == 0) {
			return;
		}
		else {
			[album setTitle:aTitle];
		}
	}
	
	if(aSummary) {
		[album setSummary:aSummary];
	}
	
	[editAlbumOperation release];
	editAlbumOperation = [[YafEditAlbumOperation alloc] initWithCatcher:self];
	[editAlbumOperation setUsername:username withToken:token];
	[editAlbumOperation updateAlbum:album setPassword:aPassword];
}

- (YafAlbum*) getAlbumWithID:(NSString*)anAlbumID
{
	for(int i = 0; i < [albums count]; i++) {
		YafAlbum* album = [albums objectAtIndex:i];
		if([[album uid] isEqualToString:anAlbumID]) {
			return album;
		}
	}
	
	return nil;
}

- (BOOL) isSignedIn
{
	return NO;
}

- (NSString*) username
{
	return username;
}

- (NSString*) password
{
	return password;
}

- (void) operationFinished:(YaOperation *)anOperation
{	
	if(!anOperation) {
		NSLog(@"Some unexpected %@ operation error.", [anOperation className]);
		return;
	}

	if([anOperation isKindOfClass:[YafSignInOperation class]]) {
		if([anOperation isSuccess]) {
			NSLog(@"Operation %@ success, user signed in with token: %@", [anOperation className], [signInOperation token]);
			[token release];
			token = [[signInOperation token] retain];
		}
		else {
			NSLog(@"Operation %@ failed with error: %@", [anOperation className], [anOperation detailedErrorMessage]);
		}

		for(id<YafCatcher> observer in observers) {
			[observer yandexFotkiObjectSyncronized:YFO_SIGN_IN operationData:nil error:[anOperation error]];
		}
	}
	else if([anOperation isKindOfClass:[YafGetAlbumsListOperation class]]) {
		if([anOperation isSuccess]) {
			NSLog(@"Operation %@ success. Albums count: %d", [anOperation className], [[getAlbumsListOperation albums] count]);
			[albums release];
			albums = [[getAlbumsListOperation albums] retain];
		}
		else {
			NSLog(@"Operation %@ failed. Reason: %@", [anOperation className], [anOperation detailedErrorMessage]);
		}
		
		for(id<YafCatcher> observer in observers) {
			[observer yandexFotkiObjectSyncronized:YFO_GET_ALBUMS_LIST operationData:nil error:[anOperation error]];
		}
	}
	else if([anOperation isKindOfClass:[YafGetAlbumOperation class]]) {
		NSDictionary* userData = nil;
		
		if([anOperation isSuccess]) {
			userData = [NSDictionary dictionaryWithObject:[getAlbumOperation album] forKey:YafFetchedAlbum];
			NSLog(@"Operation %@ success. Album description: %@", [anOperation className], [[getAlbumOperation album] description]);
		}
		else {
			NSLog(@"Operation %@ failed. Reason: %@", [anOperation className], [anOperation detailedErrorMessage]);
		}
		
		for(id<YafCatcher> observer in observers) {
			[observer yandexFotkiObjectSyncronized:YFO_GET_ALBUM operationData:userData error:[anOperation error]];
		}
	}
	else if([anOperation isKindOfClass:[YafEditAlbumOperation class]]) {
		NSDictionary* userData = nil;
		
		if([anOperation isSuccess]) {
			YafAlbum* opAlbum = [editAlbumOperation album];
			YafAlbum* updatedAlbum = [self getAlbumWithID:[opAlbum uid]];
			
			if(updatedAlbum) {
				[updatedAlbum setDataFromAlbum:opAlbum];
			}
			
			userData = [NSDictionary dictionaryWithObject:opAlbum forKey:YafFetchedAlbum];
			NSLog(@"Operation %@ success. Album description: %@", [anOperation className], [opAlbum description]);
		}
		else {
			NSLog(@"Operation %@ failed. Reason: %@", [anOperation className], [anOperation detailedErrorMessage]);
		}
		
		for(id<YafCatcher> observer in observers) {
			[observer yandexFotkiObjectSyncronized:YFO_EDIT_ALBUM operationData:userData error:[anOperation error]];
		}
	}	

	/*	 
	 //[getAlbumsListOperation setUsername:Username withToken:[signInOperation token]];
	 //[getAlbumsListOperation getAlbumsList];
	 //[createAlbumOperation setUsername:Username withToken:[signInOperation token]];
	 //[createAlbumOperation createAlbumWithTitle:[NSString stringWithFormat:@"Альбом №%d", albumsCreatedCount + 1] summary:@"Описание альбома" password:nil];
	 
	else if([anOperation isKindOfClass:[YafCreateAlbumOperation class]]) {
		if([anOperation isSuccess]) {
			NSLog(@"Operation %@ success. Album description: %@", [anOperation className], [createAlbumOperation album]);
		}
		else {
			NSLog(@"Operation %@ failed. Reason: %@", [anOperation className], [anOperation errorrMessage]);
		}
		
		albumsCreatedCount++;
		
		if(albumsCreatedCount < albumsToGenerate) {
			[createAlbumOperation createAlbumWithTitle:[NSString stringWithFormat:@"Альбом №%d", albumsCreatedCount + 1] summary:@"Описание альбома" password:nil];
		}
		else {
			albumsCreatedCount = 0;
			[signInOperation release];
			[createAlbumOperation release];
		}
	}
	else if([anOperation isKindOfClass:[YafDeleteAlbumOperation class]]) {
		if([anOperation isSuccess]) {
			NSLog(@"Operation %@ success. Albums with ID: %@ deleted", [anOperation className], [deleteAlbumOperation deletedAlbumID]);
		}
		else {
			NSLog(@"Operation %@ failed. Reason: %@", [anOperation className], [anOperation errorrMessage]);
		}
		
		// delete last album
		YafAlbum* albumToDelete = nil;
		static int currentAlbumIndex = 1;
		if([[getAlbumsListOperation albums] count] > 0 && currentAlbumIndex < [[getAlbumsListOperation albums] count]) {
			albumToDelete = [[getAlbumsListOperation albums] objectAtIndex:currentAlbumIndex];
			currentAlbumIndex++;
		}
		
		if(albumToDelete) {
			[deleteAlbumOperation setUsername:Username withToken:[signInOperation token]];
			[deleteAlbumOperation deleteAlbumWithID:[albumToDelete uid]];
		}
	}
	*/
}

- (void) fetchAlbumWithID:(NSString*)anAlbumID
{
	[getAlbumOperation release];
	getAlbumOperation = [[YafGetAlbumOperation alloc] initWithCatcher:self];
	[getAlbumOperation setUsername:username withToken:token];
	[getAlbumOperation getAlbumWithID:anAlbumID];	
}

- (NSArray*) albums
{
	return albums;
}

- (void) dealloc
{
	[username release];
	[password release];
	[token release];
	[albums release];

	[observers release];
	
	[signInOperation release];
	[createAlbumOperation release];
	[deleteAlbumOperation release];
	[getAlbumsListOperation release];

	[super dealloc];
}
@end
