//
//  YaFotkiCreateAlbumOperation.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 24.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafCreateAlbumOperation.h"
#import "YafCreateAlbumRequest.h"
#import "YafCreateAlbumRequestParser.h"

@implementation YafCreateAlbumOperation
- (void) createAlbumWithTitle:(NSString *)aTitle summary:(NSString *)aSummary password:(NSString*)aPassword
{
	// copy data
	[title release];
	title = [aTitle copy];
	
	[summary release];
	summary = [aSummary copy];
	
	[password release];
	password = [aPassword copy];

	// clear queue
	[requestsQueue removeAllObjects];
	
	// create and add album creation request
	YafCreateAlbumRequestParser* parser = [[[YafCreateAlbumRequestParser alloc] init] autorelease];
	YafCreateAlbumRequest* request = [[YafCreateAlbumRequest alloc] initWithCatcher:self parser:parser];
	[request setAlbumTitle:title];
	[request setAlbumSummary:summary];
	[request setAlbumPassword:password];
	[request setUsername:username];
	[request setToken:token];

	[requestsQueue addObject:[request autorelease]];
	
	[self start];
}

- (void) setUsername:(NSString*)aUsername withToken:(NSString*)aToken
{
	[username release];
	username = [aUsername copy];
	
	[token release];
	token = [aToken copy];
}

- (void) resetOperationData
{
	currentRequestIndex	= 0;
	
	[album release];
	album = nil;
	
	[super resetOperationData];
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
	if(!aRequest) {
		isSuccess = NO;
		[self setError:yafApiError];
		[catcher operationFinished:self];
		return;
	}
	
	NSString* errString = [NSString stringWithFormat:@"Server response code: %d. ", [aRequest responseCode]];
	
	isSuccess = NO;
	
	if([currentRequest isKindOfClass:[YafCreateAlbumRequest class]]) { 
		if([aRequest responseCode] == 201) {
			isSuccess = YES;

			[album release];
			album = [parsedData objectForKey:[YafCreateAlbumRequestParser dicKeyCreatedAlbum]];
			[album retain];
			
			if(!album) {
				isSuccess = NO;
				[self setError:yafApiError];				
			}
			
			[catcher operationFinished:self];
			return;
		}
		else if([aRequest responseCode] == 401) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Unauthorized"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 403) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Forbidden"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 400) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Bad Request or empty album title"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 404) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:[NSString stringWithFormat:@"User {%@} not found", username]]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 405) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Method Not Allowed"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 415) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Unsupported Media Type"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 500) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Internal Server Error"]];
			[self setError:serverUnreacheableError];
		}
		
		if(isSuccess == NO) {
			if(!detailedErrorMessage || [detailedErrorMessage length] == 0) {
				[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер недоступен; расширенное описание ошибки отсутствует."]];
				[self setError:serverUnreacheableError];
			}

			[catcher operationFinished:self];
		}
	}
}

- (YafAlbum*) album
{
	return album;
}

- (void) dealloc
{
	[title release];
	[summary release];
	[password release];
	[token release];
	[username release];
	[album release];
	
	[super dealloc];
}
@end
