//
//  YafDeleteAlbumOperation.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 28.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafDeleteAlbumOperation.h"
#import "YafDeleteAlbumRequest.h"

@implementation YafDeleteAlbumOperation
- (void) deleteAlbumWithID:(NSString*)anAlbumID
{
	// clear queue
	[requestsQueue removeAllObjects];
	
	[deletedAlbumID release];
	deletedAlbumID = [anAlbumID copy];
	
	// create and add album removal request
	YafDeleteAlbumRequest* request = [[YafDeleteAlbumRequest alloc] initWithCatcher:self parser:nil];
	[request setUsername:username];
	[request setToken:token];
	[request setAlbumID:anAlbumID];
	
	[requestsQueue addObject:[request autorelease]];
	
	[self start];
}

- (void) setUsername:(NSString*)aUsername withToken:(NSString*)aToken
{
	[username release];
	username = [aUsername copy];
	
	[token release];
	token = [aToken copy];
}

- (void) resetOperationData
{
	currentRequestIndex	= 0;
	
	[super resetOperationData];
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
	if(!aRequest) {
		isSuccess = NO;
		[self setError:yafApiError];
		[catcher operationFinished:self];
		return;
	}
	
	NSString* errString = [NSString stringWithFormat:@"Server response code: %d. ", [aRequest responseCode]];

	isSuccess = NO;
	
	if([currentRequest isKindOfClass:[YafDeleteAlbumRequest class]]) { 
		if([aRequest responseCode] == 204) {
			isSuccess = YES;

			[catcher operationFinished:self];
			return;
		}
		else if([aRequest responseCode] == 401) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Unauthorized"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 403) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Forbidden"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 405) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Method Not Allowed"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 500) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Internal Server Error"]];
			[self setError:serverUnreacheableError];
		}
		
		if(isSuccess == NO) {
			if(!detailedErrorMessage || [detailedErrorMessage length] == 0) {
				[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер недоступен."]];
				[self setError:serverUnreacheableError];
			}
			
			[catcher operationFinished:self];
		}
	}
}

- (NSString*) deletedAlbumID
{
	return deletedAlbumID;
}

- (void) dealloc
{
	[username release];
	[token release];
	
	[super dealloc];
}
@end
