//
//  YafEditAlbumOperation.m
//  Ya.Fotki
//
//  Created by Rim Zaidullin on 21.01.10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "YafEditAlbumOperation.h"
#import "YafEditAlbumRequest.h"
#import "YafEditAlbumRequestParser.h"

@implementation YafEditAlbumOperation
- (void) updateAlbum:(YafAlbum*)anAlbum setPassword:(NSString*)aPassword
{
	// clear queue
	[requestsQueue removeAllObjects];
	
	[album release];
	album = [anAlbum retain];

	// create and add album creation request
	YafEditAlbumRequestParser* parser = [[YafEditAlbumRequestParser alloc] init];
	YafEditAlbumRequest* request = [[YafEditAlbumRequest alloc] initWithCatcher:self parser:[parser autorelease]];
	[request setUsername:username];
	[request setToken:token];
	[request setAlbum:album];
	[request setAlbumPassword:aPassword];
	
	[requestsQueue addObject:[request autorelease]];
	
	[self start];
}

- (void) setUsername:(NSString*)aUsername withToken:(NSString*)aToken
{
	[username release];
	username = [aUsername copy];
	
	[token release];
	token = [aToken copy];
}

- (void) resetOperationData
{
	currentRequestIndex	= 0;

	[super resetOperationData];
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
	if(!aRequest) {
		isSuccess = NO;
		[self setError:yafApiError];
		[catcher operationFinished:self];
		return;
	}

	NSString* errString = [NSString stringWithFormat:@"Server response code: %d. ", [aRequest responseCode]];
	
	isSuccess = NO;
	
	if([currentRequest isKindOfClass:[YafEditAlbumRequest class]]) { 
		if([aRequest responseCode] == 200) {
			isSuccess = YES;
			
			[album release];
			album = [parsedData objectForKey:[YafEditAlbumRequestParser dicKeyAlbum]];
			[album retain];
			
			if(!album) {
				isSuccess = NO;
				[self setError:yafApiError];
			}
			
			[catcher operationFinished:self];
			
			return;
		}
		else if([aRequest responseCode] == 401) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Unauthorized"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 403) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Forbidden"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 400) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Bad Request"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 404) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:[NSString stringWithFormat:@"Album {%@} or user {%@} not found", [album uid], username]]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 415) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Unsupported Media Type"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 500) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Internal Server Error"]];
			[self setError:serverUnreacheableError];
		}
		
		if(isSuccess == NO) {
			if(!detailedErrorMessage || [detailedErrorMessage length] == 0) {
				[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер недоступен."]];
				[self setError:serverUnreacheableError];
			}
			
			[catcher operationFinished:self];
		}
	}
}

- (YafAlbum*) album
{
	return album;
}

- (void) dealloc
{
	[username release];
	[token release];
	[album release];
	
	[super dealloc];
}
@end
