//
//  YafGetAlbumOperation.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 15.01.10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "YafGetAlbumOperation.h"
#import "YafGetAlbumRequest.h"
#import "YafGetAlbumRequestParser.h"

@implementation YafGetAlbumOperation
- (void) getAlbumWithID:(NSString*)anAlbumID
{
	// clear queue
	[requestsQueue removeAllObjects];
	
	[album release];
	album = nil;
	
	[requestedAlbumID release];
	requestedAlbumID = [anAlbumID retain];
	
	// create and add album creation request
	YafGetAlbumRequestParser* parser = [[YafGetAlbumRequestParser alloc] init];
	YafGetAlbumRequest* request = [[YafGetAlbumRequest alloc] initWithCatcher:self parser:[parser autorelease]];
	[request setUsername:username];
	[request setToken:token];
	[request setAlbumID:requestedAlbumID];
	
	[requestsQueue addObject:[request autorelease]];
	
	[self start];
}

- (void) setUsername:(NSString*)aUsername withToken:(NSString*)aToken
{
	[username release];
	username = [aUsername copy];
	
	[token release];
	token = [aToken copy];
}

- (void) resetOperationData
{
	currentRequestIndex	= 0;
	
	[album release];
	album = nil;
	
	[requestedAlbumID release];
	requestedAlbumID = nil;
	
	[super resetOperationData];
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
	if(!aRequest) {
		isSuccess = NO;
		[self setError:yafApiError];
		[catcher operationFinished:self];
		return;
	}
	
	NSString* errString = [NSString stringWithFormat:@"Server response code: %d. ", [aRequest responseCode]];
	
	isSuccess = NO;
	
	if([currentRequest isKindOfClass:[YafGetAlbumRequest class]]) { 
		if([aRequest responseCode] == 200) {
			isSuccess = YES;
			
			[album release];
			album = [parsedData objectForKey:[YafGetAlbumRequestParser dicKeyAlbum]];
			[album retain];
			
			if(!album) {
				isSuccess = NO;
				[self setError:yafApiError];
			}
			
			[catcher operationFinished:self];
			
			return;
		}
		else if([aRequest responseCode] == 401) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Unauthorized"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 403) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Forbidden"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 400) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Bad Request"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 404) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:[NSString stringWithFormat:@"Album {%@} or user {%@} not found", requestedAlbumID, username]]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 500) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Internal Server Error"]];
			[self setError:serverUnreacheableError];
		}
		
		if(isSuccess == NO) {
			if(!detailedErrorMessage || [detailedErrorMessage length] == 0) {
				[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер недоступен."]];
				[self setError:serverUnreacheableError];
			}
			
			[catcher operationFinished:self];
		}
	}
}

- (YafAlbum*) album
{
	return album;
}

- (NSString*) requestedAlbumID
{
	return requestedAlbumID;
}

- (void) dealloc
{
	[username release];
	[token release];
	[album release];
	[requestedAlbumID release];
	
	[super dealloc];
}
@end