//
//  YafGetAlbumsListOperation.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 25.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafGetAlbumsListOperation.h"
#import "YafGetAlbumsListRequest.h"
#import "YafGetAlbumsListRequestParser.h"

@implementation YafGetAlbumsListOperation
- (void) getAlbumsList
{
	// clear queue
	[requestsQueue removeAllObjects];
	
	[albums release];
	albums = nil;
	
	// create and add album creation request
	YafGetAlbumsListRequestParser* parser = [[[YafGetAlbumsListRequestParser alloc] init] autorelease];
	YafGetAlbumsListRequest* request = [[YafGetAlbumsListRequest alloc] initWithCatcher:self parser:parser];
	[request setUsername:username];
	[request setToken:token];
	
	[requestsQueue addObject:[request autorelease]];

	[self start];
}

- (void) setUsername:(NSString*)aUsername withToken:(NSString*)aToken
{
	[username release];
	username = [aUsername copy];
	
	[token release];
	token = [aToken copy];
}

- (void) resetOperationData
{
	currentRequestIndex	= 0;
	
	[super resetOperationData];
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
	if(!aRequest) {
		isSuccess = NO;
		[self setError:yafApiError];
		[catcher operationFinished:self];
		return;
	}
	
	NSString* errString = [NSString stringWithFormat:@"Server response code: %d. ", [aRequest responseCode]];
	
	isSuccess = NO;
	
	if([currentRequest isKindOfClass:[YafGetAlbumsListRequest class]]) { 
		if([aRequest responseCode] == 200) {
			isSuccess = YES;
			
			if(!albums) {
				albums = [[NSArray array] retain];
			}
			
			NSArray* moreAlbums = [parsedData objectForKey:[YafGetAlbumsListRequestParser dicKeyAlbumsList]];
			NSString* nextCollectionPageURL = [parsedData objectForKey:[YafGetAlbumsListRequestParser dicKeyNextPageIncollectionURL]];
			albums = [[albums arrayByAddingObjectsFromArray:moreAlbums] retain];
			
			// we're at the last page
			if(!nextCollectionPageURL || [nextCollectionPageURL length] == 0) {
				[catcher operationFinished:self];
			}
			else {
				// clear queue
				[requestsQueue removeAllObjects];
				
				// create and add album creation request
				YafGetAlbumsListRequestParser* parser = [[[YafGetAlbumsListRequestParser alloc] init] autorelease];
				YafGetAlbumsListRequest* request = [[YafGetAlbumsListRequest alloc] initWithCatcher:self parser:parser];
				[request setUsername:username];
				[request setToken:token];
				[request setNextPageRequestUrl:nextCollectionPageURL];
				 
				[requestsQueue addObject:[request autorelease]];
				[self start];
			}
			
			return;
		}
		else if([aRequest responseCode] == 401) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Unauthorized"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 403) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Authorization error. Forbidden"]];
			[self setError:signInError];
		}
		else if([aRequest responseCode] == 400) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Bad Request"]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 404) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:[NSString stringWithFormat:@"User {%@} not found", username]]];
			[self setError:serverUnreacheableError];
		}
		else if([aRequest responseCode] == 500) {
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Internal Server Error"]];
			[self setError:serverUnreacheableError];
		}
		
		if(isSuccess == NO) {
			if(!detailedErrorMessage || [detailedErrorMessage length] == 0) {
				[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер недоступен."]];
				[self setError:serverUnreacheableError];
			}
			
			[catcher operationFinished:self];
		}
	}
}

- (NSArray*) albums
{
	return albums;
}

- (void) dealloc
{
	[username release];
	[token release];
	[albums release];
	
	[super dealloc];
}
@end
