//
//  YaFotkiSignInOperation.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 23.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafSignInOperation.h"
#import "YafAuthKeyRequestParser.h"
#import "YafAuthKeyRequest.h"
#import "YafAuthTokenRequest.h"
#import "YafAuthTokenRequestParser.h"

@implementation YafSignInOperation
- (void) signInWithLogin:(NSString *)aLogin password:(NSString *)aPassword
{
	NSLog(@"login: %@", aLogin);
	NSLog(@"password: %@", aPassword);
	
	// copy data
	login = [aLogin copy];
	password = [aPassword copy];
	
	// create and add auth key request
	YafAuthKeyRequestParser* parser = [[YafAuthKeyRequestParser alloc] init];
	YafAuthKeyRequest* request = [[YafAuthKeyRequest alloc] initWithCatcher:self parser:[parser autorelease]];
	[requestsQueue addObject:[request autorelease]];
	
	// create and add auth token request
	YafAuthTokenRequestParser* parser2 = [[YafAuthTokenRequestParser alloc] init];
	YafAuthTokenRequest* request2 = [[YafAuthTokenRequest alloc] initWithCatcher:self parser:[parser2 autorelease]];
	[requestsQueue addObject:[request2 autorelease]];
	
	[self start];
}

- (void) resetOperationData
{
	[token release];
	token = nil;
	
	[super resetOperationData];
}

- (void) requestFinished:(YaRequest *)aRequest parsedData:(NSDictionary *)parsedData
{
	if(!aRequest) {
		isSuccess = NO;
		[self setError:yafApiError];
		[catcher operationFinished:self];
		return;
	}
	
	NSString* errString = [NSString stringWithFormat:@"Server response code: %d. ", [aRequest responseCode]];
	
	// results of key request
	if([currentRequest isKindOfClass:[YafAuthKeyRequest class]]) {
		if([aRequest responseCode] != 200) {
			isSuccess = NO;
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер вернул некорректный код ответа (должен быть 200)."]];
			[self setError:serverUnreacheableError];
			
			[catcher operationFinished:self];
			return;
		}
		
		NSString* key = [parsedData objectForKey:[YafAuthKeyRequestParser dicKeyParsedDataAuthRsaKey]];
		NSString* req_id = [parsedData objectForKey:[YafAuthKeyRequestParser dicKeyRequestID]];
		
		// verify first request data
		if(!key || [key length] == 0 || !req_id || [req_id length] == 0) {
			isSuccess = NO;
			
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер вернул некорректные данные."]];
			[self setError:serverUnreacheableError];
			[catcher operationFinished:self];
			return;
		}
		
		currentRequestIndex++;
		currentRequest = [requestsQueue objectAtIndex:currentRequestIndex];
		
		NSLog(@"login 2: %@", login);
		NSLog(@"password 2: %@", password);
		
		// set next request data
		[(YafAuthTokenRequest*)currentRequest setAuthRsaKey:key];
		[(YafAuthTokenRequest*)currentRequest setAuthRequestID:req_id];
		[(YafAuthTokenRequest*)currentRequest setAuthLogin:login];
		[(YafAuthTokenRequest*)currentRequest setAuthPassword:password];
		
		[currentRequest start];
	}
	else if([currentRequest isKindOfClass:[YafAuthTokenRequest class]]) {
		NSString* aToken = [parsedData objectForKey:[YafAuthTokenRequestParser dicKeyTokenID]];
		if([aRequest responseCode] == 403) {
			isSuccess = NO;
			
			NSString* parsedErrorMessageFromServer = [parsedData objectForKey:[YafAuthTokenRequestParser dicKeyErrorMessage]];
			[self setDetailedErrorMessage:[errString stringByAppendingString:parsedErrorMessageFromServer]];
			[self setError:signInError];
			
			[catcher operationFinished:self];
			return;
		}
		else if([aRequest responseCode] == 400) {
			isSuccess = NO;
			
			NSString* parsedErrorMessageFromServer = [parsedData objectForKey:[YafAuthTokenRequestParser dicKeyErrorMessage]];
			[self setDetailedErrorMessage:[errString stringByAppendingString:parsedErrorMessageFromServer]];
			[self setError:signInError];
			 
			[catcher operationFinished:self];
			return;
		}
		
		// verify first request data
		if(!aToken || [aToken length] == 0) {
			isSuccess = NO;
			
			[self setDetailedErrorMessage:[errString stringByAppendingString:@"Сервер вернул некорректные данные."]];
			[self setError:serverUnreacheableError];
			
			[catcher operationFinished:self];
			return;
		}
		
		[token release];
		token = [aToken copy];
		isSuccess = YES;
		[catcher operationFinished:self];
	}
}

- (NSString*) token
{
	return token;
}

- (void) dealloc
{
	[login release];
	[password release];
	[token release];
	
	[super dealloc];
}
@end
