//
//  YafCreateAlbumRequestParser.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 25.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafCreateAlbumRequestParser.h"


@implementation YafCreateAlbumRequestParser
- (NSDictionary*) parseData:(NSData*)data
{
	NSString* xmlString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
	[xmlString autorelease];
	if(!xmlString || [xmlString length] == 0) {
		return nil;
	}
	
	NSError *errmsg;
	NSXMLDocument *xmlDoc = [[NSXMLDocument alloc] initWithXMLString:xmlString options:NSXMLDocumentXMLKind error:&errmsg];
	[xmlDoc autorelease];
	
	if(!xmlDoc) {
		return nil;
	}
	
	NSString* name = [[xmlDoc rootElement] name];
	if([name isEqualToString:@"entry"] == NO) {
		return nil;
	}
	
	album = [YafCreateAlbumRequestParser parseAlbumEntry:[xmlDoc rootElement]];
	[album retain];
	
	return [NSDictionary dictionaryWithObject:album forKey:[YafCreateAlbumRequestParser dicKeyCreatedAlbum]];
}

+ (YafAlbum*) parseAlbumEntry:(NSXMLElement*)albumXMLElement
{	
	YafAlbum* anAlbum = [[YafAlbum alloc] init];
	NSArray* elements = [albumXMLElement children];
	
	// flags for found and parsed nodes
	BOOL uidProcessed = NO;
	BOOL titleProcessed = NO;
	BOOL linkProcessedSelf = NO;
	BOOL linkProcessedEdit = NO;
	BOOL linkProcessedPhotos = NO;
	BOOL linkProcessedAlternate = NO;
	BOOL protectedProcessed = NO;
	BOOL imageCountProcessed = NO;
	
	int i;
	for(i = 0; i < [elements count]; i++) {
		NSXMLElement* element = [elements objectAtIndex:i];
		
		if([[element name] isEqualToString:@"id"]) {
			NSArray* components = [[element stringValue] componentsSeparatedByString:@":"];
			
			// parsing error occured!
			if([components count] == 0) {
				continue;
			}
			
			uidProcessed = YES;
			[anAlbum setUid:[components objectAtIndex:[components count] - 1]];
		}
		else if([[element name] isEqualToString:@"title"]) {
			titleProcessed = YES;
			[anAlbum setTitle:[element stringValue]];
		}
		else if([[element name] isEqualToString:@"summary"]) {
			[anAlbum setSummary:[element stringValue]];
		}
		else if([[element name] isEqualToString:@"link"]) {
			NSArray* attributes = [element attributes];
			NSString* rel = nil;
			NSString* href = nil;
			
			int j;
			for(j = 0; j < [attributes count]; j++) {
				NSXMLNode* node = [attributes objectAtIndex:j];
				
				if([[node name] isEqualToString:@"rel"]) {
					rel = [node stringValue];
				}
				else if([[node name] isEqualToString:@"href"]) {
					href = [node stringValue];
				}		
			}
			
			if(rel) {
				if([rel isEqualToString:@"self"] && href && [href length] > 0) {
					linkProcessedSelf = YES;
					[anAlbum setLinkSelf:href];
				}
				else if([rel isEqualToString:@"edit"] && href && [href length] > 0) {
					linkProcessedEdit = YES;
					[anAlbum setLinkEdit:href];
				}
				else if([rel isEqualToString:@"photos"] && href && [href length] > 0) {
					linkProcessedPhotos = YES;
					[anAlbum setLinkPhotos:href];
				}
				else if([rel isEqualToString:@"alternate"] && href && [href length] > 0) {
					linkProcessedAlternate = YES;
					[anAlbum setLinkAlternate:href];
				}
			}
		}
		else if([[element name] isEqualToString:@"published"]) {
			if([element stringValue] && [[element stringValue] length] > 0) {
				[anAlbum setDatePublished:[element stringValue]];
			}
		}
		else if([[element name] isEqualToString:@"app:edited"]) {
			if([element stringValue] && [[element stringValue] length] > 0) {
				[anAlbum setDateEdited:[element stringValue]];
			}
		}
		else if([[element name] isEqualToString:@"updated"]) {
			if([element stringValue] && [[element stringValue] length] > 0) {
				[anAlbum setDateUpdated:[element stringValue]];
			}
		}
		else if([[element name] isEqualToString:@"f:protected"]) {
			NSArray* attributes = [element attributes];
			
			int j;
			for(j = 0; j < [attributes count]; j++) {
				NSXMLNode* node = [attributes objectAtIndex:j];
				
				if([[node name] isEqualToString:@"value"]) {
					if([[node stringValue] isEqualToString:@"false"]) {
						protectedProcessed = YES;
						[anAlbum setIsProtected:NO];
					}
					else if([[node stringValue] isEqualToString:@"true"]) {
						protectedProcessed = YES;
						[anAlbum setIsProtected:YES];
					}
				}
			}
		}
		else if([[element name] isEqualToString:@"f:image-count"]) {
			NSArray* attributes = [element attributes];
			
			int j;
			for(j = 0; j < [attributes count]; j++) {
				NSXMLNode* node = [attributes objectAtIndex:j];
				
				if([[node name] isEqualToString:@"value"] && [[node stringValue] length] > 0) {
					NSInteger intVal = [[node stringValue] integerValue];
					imageCountProcessed = YES;
					[anAlbum setImageCount:intVal];
				}
			}
		}
	}
	
	//summaryProcessed -- summary may be missing, it's ok
	if(!uidProcessed || !titleProcessed || !linkProcessedSelf || !linkProcessedEdit || !linkProcessedPhotos ||
	   !linkProcessedAlternate || !protectedProcessed || !imageCountProcessed) {
		[anAlbum release];
		return nil;
	}
	
	return [anAlbum autorelease];
}

- (YafAlbum*) album
{
	return album;
}

+ (NSString*) dicKeyCreatedAlbum
{
	return @"created_album";
}

- (void) dealloc
{
	[album release];
	
	[super dealloc];
}
@end