//
//  YaFotkiAuthorisationTokenRequest.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 23.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafAuthTokenRequest.h"
#import "YaStringAdditions.h"
#import "cp_rsa_c_wrapper.h"

@implementation YafAuthTokenRequest
- (id) initWithCatcher:(id<YaRequestCatcher>)aCatcher parser:(id<YaRequestParser>)aParser
{
	if(self = [super initPostRequest]) {
		[self setParser:aParser];
		[self setCatcher:aCatcher];

		RequestUrl = @"http://auth.mobile.yandex.ru/yamrsa/token/";

		[request setURL:[NSURL URLWithString:RequestUrl]];
		[request addRequestHeader:@"Host" value:@"auth.mobile.yandex.ru"];
		[request addRequestHeader:@"Content-Type" value:@"application/x-www-form-urlencoded"];
		[request setTimeOutSeconds:10];
	}
	
	return self;
}

- (BOOL) isInputDataOk
{
	BOOL retVal = YES;
	
	if(!authLogin || [authLogin length] == 0) {
		retVal = NO;
	}

	if(!authPassword || [authPassword length] == 0) {
		retVal = NO;
	}

	if(!authRequestID || [authRequestID length] == 0) {
		retVal = NO;
	}
	
	if(!authRsaKey || [authRsaKey length] == 0) {
		retVal = NO;
	}
	
	return retVal;
}

- (void) start
{
	if([self isInputDataOk] == NO) {
		localErrorMessage = @"Malformed request input data!";
		[catcher requestFailed:self];
		return;
	}
	
	// crypt [rsa key request id] and credentials with rsa key
	NSMutableData *postBodyData = nil;
	@try {
		char tmp_buf[1024];
		NSString *xmlEncodedLogin = [NSString encodeStringForXML:authLogin];
		NSString *xmlEncodedPassword = [NSString encodeStringForXML:authPassword];
		NSString *auth = [NSString stringWithFormat:@"<credentials login=\"%@\" password=\"%@\"/>", xmlEncodedLogin, xmlEncodedPassword];
		
		rsa_encrypt_text([authRsaKey UTF8String], [auth UTF8String], tmp_buf);
		NSString *userCredentials = [NSString encodeStringForURL:([NSString stringWithUTF8String:tmp_buf])];
		NSString *postBodyString = [NSString stringWithFormat:@"request_id=%@&credentials=%@", authRequestID, userCredentials];
		
		postBodyData = [NSMutableData data];
		[postBodyData appendData:[postBodyString dataUsingEncoding:NSUTF8StringEncoding]];
	}
	@catch(NSException *ex) {
		NSLog(@"[%@ start] Caught %@: %@", [self className], [ex name], [ex reason]);
		localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
		[localErrorMessage retain];
		[catcher requestFailed:self];
		return;
	}
	
	if(!postBodyData) {
		NSLog(@"[%@ start] Error: unable to form post data", [self className]);
		localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
		[localErrorMessage retain];
		[catcher requestFailed:self];
		return;
	}
		
	// request authentification token
	[request setPostBody:postBodyData];

	[super start];
}

- (void) dealloc
{
	[authRsaKey release];
	[authRequestID release];
	[authLogin release];
	[authPassword release];
	
	[super dealloc];
}

@synthesize authRsaKey;
@synthesize authRequestID;
@synthesize authLogin;
@synthesize authPassword;
@end
