//
//  YaFotkiAuthorisationCreateAlbumRequest.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 24.12.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "YafCreateAlbumRequest.h"
#import "YaStringAdditions.h"

@implementation YafCreateAlbumRequest
- (id) initWithCatcher:(id<YaRequestCatcher>)aCatcher parser:(id<YaRequestParser>)aParser
{
	if(self = [super initPostRequest]) {
		[self setParser:aParser];
		[self setCatcher:aCatcher];
				
		[request addRequestHeader:@"Host" value:@"api-fotki.yandex.ru"];
		[request addRequestHeader:@"Content-Type" value:@"application/atom+xml; charset=utf-8; type=entry"];
		[request setTimeOutSeconds:10];
	}
	
	return self;
}

- (BOOL) isInputDataOk
{
	BOOL retVal = YES;
	
	if(!albumTitle || [albumTitle length] == 0) {
		retVal = NO;
	}
	
	if(!username || [username length] == 0) {
		retVal = NO;
	}

	if(!token || [token length] == 0) {
		retVal = NO;
	}
	
	return retVal;
}
	
- (void) start
{
	if([self isInputDataOk] == NO) {
		localErrorMessage = @"Malformed request input data!";
		[catcher requestFailed:self];
		return;
	}
	
	RequestUrl = [NSString stringWithFormat:@"http://api-fotki.yandex.ru/api/users/%@/albums/", username];
	[request setURL:[NSURL URLWithString:RequestUrl]];
	
	NSString* authHeader = [NSString stringWithFormat:@"FimpToken realm=\"fotki.yandex.ru\", token=\"%@\"", token];
	[request addRequestHeader:@"Authorization" value:authHeader];
	
	NSMutableData *postBodyData = nil;
	@try {
		NSMutableString *postBodyString = [[[NSMutableString alloc] init] autorelease];
		[postBodyString appendString:@"<entry xmlns=\"http://www.w3.org/2005/Atom\" xmlns:f=\"yandex:fotki\">\n"];
		[postBodyString appendFormat:@"<title>%@</title>\n", albumTitle];
		[postBodyString appendFormat:@"<summary>%@</summary>\n", albumSummary];
		
		if(albumPassword && [albumPassword length] > 0) {
			[postBodyString appendFormat:@"<f:password>%@</f:password>\n", albumPassword];
		}
		
		[postBodyString appendFormat:@"</entry>"];
	
		//NSLog(postBodyString);
		//NSString *encodedPostBodyString = [NSString encodeStringForURL:postBodyString];
		
		postBodyData = [NSMutableData data];
		[postBodyData appendData:[postBodyString dataUsingEncoding:NSUTF8StringEncoding]];
	}
	@catch(NSException *ex) {
		NSLog(@"[%@ start] Caught %@: %@", [self className], [ex name], [ex reason]);
		localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
		[localErrorMessage retain];
		[catcher requestFailed:self];
		return;
	}
	
	if(!postBodyData) {
		NSLog(@"[%@ start] Error: unable to form post data", [self className]);
		localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
		[localErrorMessage retain];
		[catcher requestFailed:self];
		return;
	}
	
	// send "create album" request
	[request setPostBody:postBodyData];
	
	[super start];
}

- (void) dealloc
{
	[albumTitle release];
	[albumSummary release];
	[albumPassword release];
	[username release];
	[token release];
	
	[super dealloc];
}

@synthesize albumTitle;
@synthesize albumSummary;
@synthesize albumPassword;
@synthesize username;
@synthesize token;
@end