//
//  YafEditAlbumRequest.m
//  FotkiUploader v2.0 v2
//
//  Created by Rim Zaidullin on 16.01.10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "YafEditAlbumRequest.h"


@implementation YafEditAlbumRequest
- (id) initWithCatcher:(id<YaRequestCatcher>)aCatcher parser:(id<YaRequestParser>)aParser
{
	if(self = [super initPutRequest]) {
		[self setParser:aParser];
		[self setCatcher:aCatcher];
		
		[request addRequestHeader:@"Host" value:@"api-fotki.yandex.ru"];
		[request addRequestHeader:@"Content-Type" value:@"application/atom+xml; type=entry"];
		[request setTimeOutSeconds:10];
	}

	return self;
}

- (BOOL) isInputDataOk
{
	BOOL retVal = YES;
	
	if(!username || [username length] == 0) {
		retVal = NO;
	}
	
	if(!token || [token length] == 0) {
		retVal = NO;
	}

	if(![album uid] || [[album uid] length] == 0) {
		retVal = NO;
	}

	if(![album title] || [[album title] length] == 0) {
		retVal = NO;
	}

	return retVal;
}

- (void) start
{
	if([self isInputDataOk] == NO) {
		localErrorMessage = @"Malformed request input data!";
		[catcher requestFailed:self];
		return;
	}
	
	RequestUrl = [NSString stringWithFormat:@"http://api-fotki.yandex.ru/api/users/%@/album/%@/", username, [album uid]];
	[request setURL:[NSURL URLWithString:RequestUrl]];	
	
	NSString* authHeader = [NSString stringWithFormat:@"FimpToken realm=\"fotki.yandex.ru\", token=\"%@\"", token];
	[request addRequestHeader:@"Authorization" value:authHeader];
	
	NSMutableData* postBodyData = nil;

	@try {
		NSString* postBodyString = [album xmlStringValueWithUsername:username albumPassword:albumPassword];

		if(!postBodyString) {
			NSLog(@"[%@ start] Generated error!");
			localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
			[localErrorMessage retain];
			[catcher requestFailed:self];
			return;			
		}		
	
		postBodyData = [NSMutableData data];
		[postBodyData appendData:[postBodyString dataUsingEncoding:NSUTF8StringEncoding]];
	}
	@catch(NSException *ex) {
		NSLog(@"[%@ start] Caught %@: %@", [self className], [ex name], [ex reason]);
		localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
		[localErrorMessage retain];
		[catcher requestFailed:self];
		return;
	}
	
	if(!postBodyData) {
		NSLog(@"[%@ start] Error: unable to form post data", [self className]);
		localErrorMessage = [NSString stringWithFormat:@"%@ data could not be created!", [self className]];
		[localErrorMessage retain];
		[catcher requestFailed:self];
		return;
	}
	
	// send "create album" request
	[request setPostBody:postBodyData];
	
	[super start];
}

- (void) dealloc
{
	[album release];
	[username release];
	[token release];
	[albumPassword release];
	
	[super dealloc];
}

@synthesize album;
@synthesize albumPassword;
@synthesize username;
@synthesize token;
@end
